package com.lemon.lifecenter.mapper;

import java.util.HashMap;

import org.apache.ibatis.annotations.Mapper;
import org.springframework.stereotype.Repository;

import com.lemon.lifecenter.dto.LoginDTO;

@Repository
@Mapper
public interface LoginMapper {
    public int selectMemberCount(LoginDTO dto);
    public LoginDTO selectMemberData(LoginDTO dto);
    public Integer updateMemberLoginData(LoginDTO dto);
    public int selectMemberIdCount(LoginDTO dto);
    public int selectMemberFailCount(LoginDTO dto);
    public void insertAccessHistory(HashMap<String, String> hashMap);
}
